#!/usr/bin/env python

import functools
import sys

import click


class color:
    PURPLE = "\033[95m"
    CYAN = "\033[96m"
    DARKCYAN = "\033[36m"
    BLUE = "\033[94m"
    GREEN = "\033[92m"
    YELLOW = "\033[93m"
    RED = "\033[91m"


class markup:
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"
    END = "\033[0m"


info = markup.BOLD + color.PURPLE + "[STATUS] " + markup.END
usage = markup.BOLD + color.YELLOW + "[USAGE] " + markup.END
fail = markup.BOLD + color.RED + "[ERROR] " + markup.END
warning = markup.BOLD + color.YELLOW + "[WARNING] " + markup.END


def echov(text, verbose=True, nl=True):
    """ Wraps around click.echo and expands it by a `verbose` option.

    Parameter
    ---------
    text: str
        Text that is printed to standard output.
    verbose: {True, False}, optional
        Determines whether something is printed at all.
    nl: {True, False}, optional
        If set, a new line is added to the output.
    """
    if verbose:
        click.echo(info + text, nl=nl)


def echoe(text):
    """ Prints text to error output using click.echo and exits.

    Parameter
    ---------
    text: str
        Text that is printed to error output.
    """
    click.echo(fail + text, err=True)
    sys.exit(1)



#!/usr/bin/env python

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from ratelimit import limits, sleep_and_retry

from ytscraper.helper.echo import echoe


def get_youtube_handle(api_key):
    """ Returns the YouTube Data API v3 handle.

    This method returns the handle for the YouTube Data API v3 
    in order to request data.

    Parameter
    ---------
    api_key: str
        An authorized API key. 
    """
    try:
        return build("youtube", "v3", developerKey=api_key)
    except HttpError:
        echoe(
            """ There was an error while connecting to the YouTube API.
        Please check your API key."""
        )


@sleep_and_retry
@limits(3_000_000, 100)
def check_rate_limit():
    """ Sentinel method that limits the requests per 100 seconds. """
    pass


# TODO relevanceLanguage parameter
def get_search_videos(handle, query, maxResults, regionCode="de", safeSearch="none"):
    check_rate_limit()
    response = (
        handle.search()
        .list(
            q=query,
            part="id",
            type="video",
            maxResults=maxResults,
            safeSearch=safeSearch,  # none, moderate, strict
            regionCode=regionCode,
        )
        .execute()
    )
    return list(map(lambda l: l["id"]["videoId"], response["items"]))


# TODO Why is related videos so weird?
def get_related_videos(handle, videoId, maxResults, regionCode="de", safeSearch="none"):
    check_rate_limit()
    response = (
        handle.search()
        .list(
            relatedToVideoId=videoId,
            part="id",
            type="video",
            maxResults=maxResults + 1,  # relatedToVideoId returns 1 less
            safeSearch=safeSearch,  # none, moderate, strict
            regionCode=regionCode,
        )
        .execute()
    )
    video_ids = list(map(lambda l: l["id"]["videoId"], response["items"]))
    return video_ids[:maxResults]



#!/usr/bin/env python

from collections import deque
from pprint import pprint
from urllib import parse

import click

from ytscraper.helper.config import update_config
from ytscraper.helper.echo import echoe, echov
from ytscraper.helper.node import convert_to_node_list
from ytscraper.helper.yt_api import get_search_videos, get_youtube_handle


@click.command()
@click.argument(
    "search-type", default="query", type=click.Choice(["term", "url", "id"])
)
@click.argument("query", nargs=1, required=True)
@click.option(
    "--number",
    "-n",
    multiple=True,
    type=click.IntRange(1, 50),
    help="Number of videos fetched per level.",
)
@click.option("--depth", "-d", type=int, help="Number of recursion steps.")
@click.option("--api-key", "-k", type=str, help="API Key to use YouTube API v3.")
@click.pass_context
def search(context, search_type, query, **options):
    """Searches YouTube using a specified query."""
    config = context.obj

    # CONFIGURATION
    echov("Updating configuration with command line options.", config["verbose"])
    update_config(config, options)
    echov("Done! Working with the following configuration:", config["verbose"])
    if config["verbose"]:
        pprint(config)

    # AUTHENTICATION
    echov("Starting YouTube authentication.", config["verbose"])
    if "api_key" not in config:
        echoe(
            """You need to provide an API key using `--api-key` 
        or the configuration file in order to query YouTube's API.
        Please see README on how to obtain such a key."""
        )
    handle = get_youtube_handle(config["api_key"])
    echov("API access established.", config["verbose"])

    # ARGUMENT PARSING
    if search_type == "term":
        echov("Starting search using query {query}.", config["verbose"])
        start_ids = get_search_videos(handle, query, config["number"][0])
    elif search_type == "id":
        echov("Starting search using video id {query}.", config["verbose"])
        start_ids = [query]
    elif search_type == "url":
        echov("Starting search using the following video url:", config["verbose"])
        echov(query, config["verbose"])
        qterm = parse.urlsplit(query).query
        video_id = parse.parse_qs(qterm)["v"]
        start_ids = [video_id]

    # QUERY
    node_list = convert_to_node_list(handle, start_ids, 0, config["number"])
    node_queue = deque(node_list)
    processed_nodes = []
    while True:
        if len(node_queue) == 0:
            break
        node = node_queue.popleft()
        processed_nodes.append(node)
        echov("Current Video: " + node.videoId, config["verbose"])
        if node.depth < config["depth"]:
            # Clamp level_branch index to specified branch array.
            new_nodes = convert_to_node_list(
                handle, node.relatedVideos, node.depth + 1, config["number"]
            )
            node_queue.extend(new_nodes)

    echov("Query finished! Result:")
    for node in processed_nodes:
        pprint(node)

    # TODO Save to file



from pprint import pprint

import click

from ytscraper.commands import fetch, search
from ytscraper.helper.config import load_config, update_config
from ytscraper.helper.echo import echov


@click.group()
@click.option("--config-path", "-c", type=click.Path(), help="YAML configuration file.")
@click.option("--verbose", "-v", is_flag=True, help="Show more verbose output.")
@click.pass_context
def run(context, config_path, verbose):
    echov("Reading configuration file.", verbose)
    context.obj = {}
    update_config(context.obj, load_config(config_path))
    echov("Read the following configuration:", verbose)
    if verbose:
        pprint(context.obj)

    context.obj["verbose"] = verbose


run.add_command(search.search)
run.add_command(fetch.fetch)




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ytscraper', 'ytscraper.commands', 'ytscraper.helper']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.3,<2.0.0',
 'click>=7.1.1,<8.0.0',
 'google-api-python-client>=1.8.0,<2.0.0',
 'ratelimit>=2.2.1,<3.0.0',
 'toml>=0.10.0,<0.11.0',
 'wrapt>=1.12.1,<2.0.0']

entry_points = \
{'console_scripts': ['poetry = yt-scraper.api:entry_point']}

setup_kwargs = {
    'name': 'yt-scraper',
    'version': '0.3.2',
    'description': 'Command line utility querying the YouTube API v3.',
    'long_description': 'fetch-yt.py\n\n`-q --query STRING` \nText that is used for the search on YouTube. Needed\n\n`-o --output`\nSpecifies the output file.\n**Default:** ``data/result.sql``\n\n`-n --number INTEGER (ARRAY)`\nNumber of videos fetched per level. \nIf an array is provided, the i-th element equals \nthe number of videos fetched on the i-th level.\n**Default**: 10\n\n`-l --levels INTEGER`\nNumber of recursion steps per video.\n**Default**: 1\n\n`-v --verbose`\nPrint more information to output.\n\n\n## Known Issues\n\n- The `number` parameter is restricted to maximal 50. This can be fixed in the\nfuture by iterating through result pages.\n',
    'author': 'Michael Brauweiler',
    'author_email': 'michael.brauweiler@posteo.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python

from typing import NamedTuple, Tuple

from ytscraper.helper.yt_api import get_related_videos


class VideoNode(NamedTuple):
    videoId: str
    depth: int
    rank: int
    relatedVideos: Tuple


# TODO make it more obvious that number is only a maximal value
def build_node(handle, video_id, rank, depth, related_number):
    related_videos = get_related_videos(handle, video_id, related_number)
    video_node = VideoNode(video_id, depth, rank, tuple(related_videos))
    return video_node


def get_clamp_index(container, index):
    clamped_index = max(0, min(index, len(container) - 1))
    return container[clamped_index]


def convert_to_node_list(handle, video_ids, depth, numbers):
    # Related videos are one step deeper
    related_number = get_clamp_index(numbers, depth + 1)
    video_nodes = []
    for rank, video_id in enumerate(video_ids):
        node = build_node(handle, video_id, rank, depth, related_number)
        video_nodes.append(node)
    return video_nodes
